@@ -13,10 +13,13 @@
 //! * Term frequency - inverse document frequency count vectorization
 //! * Whitening
 
-pub mod count_vectorization;
+mod countgrams;
 pub mod error;
 mod helpers;
 pub mod linear_scaling;
 pub mod norm_scaling;
 pub mod tf_idf_vectorization;
 pub mod whitening;
+
+pub use countgrams::{CountVectorizer, CountVectorizerParams, CountVectorizerValidParams};
+pub use error::{PreprocessingError, Result};
